@@ -4,14 +4,15 @@
 
 #include <assert.h>
 #include <errno.h>
+
 //#define _GNU_SOURCE         /* See feature_test_macros(7) */
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#include <string.h> //memcpy, etc
+#include <string.h> //memcpy, strlen, etc
 #include <sys/types.h>
-#include <sys/param.h> /* for MAXHOSTNAMELEN */
-#include <unistd.h> /* close() and others */
+#include <sys/param.h> //* for MAXHOSTNAMELEN */
+#include <unistd.h>    //* close() and others */
 
 #include <poll.h>
 #include <sys/socket.h>
@@ -19,11 +20,30 @@
 #include <netdb.h>
 #include <pthread.h>  // for MacOS
 
+
+#include <numeric>
+#include <string>
+#include <mutex>
+#include <chrono>
+using namespace std::chrono;
+namespace chrono = std::chrono;
+
+#include <cassert>
+#include <stdexcept>  //for std::exception
+#include <map>
+#include <stack>
+
+// for async
+#include <future>
+
+//#include <unordered_map>
+// #include <iterator>
+
 //#include <semaphore.h>
 
 #ifdef __linux__
 #   include <endian.h> // network endian is "be".
-#else
+#else // i.e. macOSX
 inline u_int64_t htobe64(u_int64_t q) {
   return (htonl(q>>32) +((u_int64_t)htonl(q & 0x00000000ffffffff)<<32)) ;
 };
@@ -47,29 +67,76 @@ template<class T> struct Property {
 };
    
 namespace nsHiSLIP{
+  typedef class HiSLIP_FatalError:std::runtime_error {
+  public:
+    HiSLIP_FatalError(const std::string& msg):std::runtime_error(msg){};
+    HiSLIP_FatalError(const char * msg):std::runtime_error(msg){
+    }
+  } FatalError_t;
+  
+  typedef class HiSLIP_Error:std::exception {
+    const char *msg;
+  public:
+    HiSLIP_Error(const char * const msg){
+      this->msg=msg;
+    }
+  } Error_t;
+  
+  typedef class HiSLIP_Interrupted:std::exception {
+    const char *msg;
+  public:
+    HiSLIP_Interrupted(const char * const msg){
+      this->msg=msg;
+    }
+  } Interrupted_t;
+  
+  typedef class HiSLIP_SRQ:std::exception {
+    const char *msg;
+  public:
+    HiSLIP_SRQ(const char * const msg){
+      this->msg=msg;
+    }
+  } SRQ_t;
+  
+  typedef class HiSLIP_TimeoutError:std::runtime_error{
+  public:
+    HiSLIP_TimeoutError(const std::string& msg):std::runtime_error(msg){};
+    HiSLIP_TimeoutError(const char * msg):std::runtime_error(msg){}
+  } TimeoutError_t;
+  
+  typedef class HiSLIP_RuntimeError:std::runtime_error{
+  public:
+    HiSLIP_RuntimeError(const std::string& msg):std::runtime_error(msg){};
+    HiSLIP_RuntimeError(const char * msg):std::runtime_error(msg){}
+  } RuntimeoutError_t;
   
   //constants
   typedef enum CC_reuqest{
-			  RemoteDisable=0,
-			  RemoteEnable=1,
-			  RemoteDisableGTL=2, // disable remote  and goto local
-			  RemoteEnableGTR=3, // Enable remote and goto remote
-			  RemoteEnableLLO=4, // Enable remote and lock out local
-			  RemoteEnableGTRLLO=5, //
-			  RTL=6
+    RemoteDisable=0,
+    RemoteEnable=1,
+    RemoteDisableGTL=2, // disable remote  and goto local
+    RemoteEnableGTR=3, // Enable remote and goto remote
+    RemoteEnableLLO=4, // Enable remote and lock out local
+    RemoteEnableGTRLLO=5, //
+    RTL=6
   } CC_request_t;
   
   typedef enum CC_Lock{
-		       release =0,
-		       request =1
+    release =0,
+    request =1
   } CC_Lock_t;
 
-  typedef enum CC_LockResponse{
-			       fail=0,         //Lock was requested but not granted
-			       success=1,      //release of exclusive lock
-			       success_shared=2, //release of shared lock
-			       error=3 // Invalide
-  } CC_LockResponse_t;
+  typedef enum CC_LockRequestResponse{
+    fail=0,         //Lock was requested but not granted
+    success=1,      //release of exclusive lock
+    locK_error=3 // Invalid
+  } CC_LockRequestResponse_t;
+  
+  typedef enum CC_LockReleaseResponse{
+    success_exclusive=1,      //release of exclusive lock
+    success_shared=2, //release of shared lock
+    release_error=3 // Invalid
+  } CC_LockReleaseResponse_t;
 
   static const long  PROTOCOL_VERSION_MAX = 0x7f7f ; // # <major><minor> = <1><1> that is 257
   static const long  INITIAL_MESSAGE_ID = 0xffffff00 ;
@@ -88,60 +155,62 @@ namespace nsHiSLIP{
   static const char  Prologue[]={'H','S'};
   //
   typedef enum Message_Types{
-			     Initialize = 0,
-			     InitializeResponse = 1,
-			     FatalError = 2,
-			     Error = 3,
-			     AsyncLock = 4,
-			     AsyncLockResponse = 5,
-			     Data = 6,
-			     DataEnd = 7,
-			     DeviceClearComplete = 8,
-			     DeviceClearAcknowledge = 9,
-			     AsyncRemoteLocalControl = 10,
-			     AsyncRemoteLocalResponse = 11,
-			     Trigger = 12,
-			     Interrupted = 13,
-			     AsyncInterrupted = 14,
-			     AsyncMaximumMessageSize = 15,
-			     AsyncMaximumMessageSizeResponse = 16,
-			     AsyncInitialize = 17,
-			     AsyncInitializeResponse = 18,
-			     AsyncDeviceClear = 19,
-			     AsyncServiceRequest = 20,
-			     AsyncStatusQuery = 21,
-			     AsyncStatusResponse = 22,
-			     AsyncDeviceClearAcknowledge = 23,
-			     AsyncLockInfo = 24,
-			     AsyncLockInfoResponse = 25,
-			     // for HiSLIP ver2
-			     GetDescriptors =26,
-			     GetDescriptorsResponse =27,
-			     StartTLS = 28,
-			     AsyncStartTLS = 29,
-			     AsyncStartTLSResponse = 30,
-			     EndTLS = 31,
-			     AsyncEndTLS = 32,
-			     AsyncEndTLSResponse = 33,
-			     GetSaslMechanismList = 34,
-			     GetSaslMechanismListResponse = 35,
-			     AuthenticationStart = 36,
-			     AuthenticationExchange = 37,
-			     AuthenticationResult = 38,
-			     // 39-127 are reserved for future use.
-			     // I don't watn to use negative value to
-			     // represent ANY message.
-			     // So I picked 127 from reserved values
-			     // for this purpose.
-			     AnyMessages=127 // 128-255 are reserved for vendor use.
-  } Message_Types_t;
-
+    Initialize = 0, //  SC
+    InitializeResponse = 1, //SR
+    FatalError = 2, // E
+    Error = 3, // E
+    AsyncLock = 4,// AC
+    AsyncLockResponse = 5,   //AR
+    Data = 6,
+    DataEnd = 7,
+    DeviceClearComplete = 8,  
+    DeviceClearAcknowledge = 9,
+    AsyncRemoteLocalControl = 10, //AC
+    AsyncRemoteLocalResponse = 11, //AR
+    Trigger = 12,
+    Interrupted = 13,
+    AsyncInterrupted = 14,
+    AsyncMaximumMessageSize = 15,  //AC
+    AsyncMaximumMessageSizeResponse = 16, //AR
+    AsyncInitialize = 17, //AC
+    AsyncInitializeResponse = 18, //AR
+    AsyncDeviceClear = 19, //AC
+    AsyncServiceRequest = 20, 
+    AsyncStatusQuery = 21, //AC
+    AsyncStatusResponse = 22, //AR
+    AsyncDeviceClearAcknowledge = 23, 
+    AsyncLockInfo = 24, //AC
+    AsyncLockInfoResponse = 25, //AR
+    // for HiSLIP ver2
+    GetDescriptors =26,
+    GetDescriptorsResponse =27,
+    StartTLS = 28,
+    AsyncStartTLS = 29,
+    AsyncStartTLSResponse = 30,
+    EndTLS = 31,
+    AsyncEndTLS = 32,
+    AsyncEndTLSResponse = 33,
+    GetSaslMechanismList = 34,
+    GetSaslMechanismListResponse = 35,
+    AuthenticationStart = 36,
+    AuthenticationExchange = 37,
+    AuthenticationResult = 38,
+    // 39-127 are reserved for future use.
+    // I don't watn to use negative value to
+    // represent ANY message.
+    // So I picked 127 from reserved values
+    // for this purpose.
+    AnyMessages=127 // 128-255 are reserved for vendor use.
+  } _Message_Types_t;
+  
+  typedef u_int8_t Message_Types_t;
+  
   typedef enum Error_code{
-			 UnidentifiedError,
-			 UnrecognizedMessageType,
-			 UnrecognizedControlCode,
-			 UnrecognizedVendorDefinedMessage,
-			 MessageTooLarge
+    UnidentifiedError,
+    UnrecognizedMessageType,
+    UnrecognizedControlCode,
+    UnrecognizedVendorDefinedMessage,
+    MessageTooLarge
   } Error_code_t;
   static const char *Error_Messages[] =
     {
@@ -152,11 +221,11 @@ namespace nsHiSLIP{
      "Message too large"
     };
   typedef enum Fatal_Error_code {
-				 UnidentifiedFatalError,
-				 PoorlyFormedMmessageHeader,
-				 AttemptToUseConnectionWithoutBothChannels,
-				 InvalidInitializationSequence,
-				 ServerRefusedConnection
+    UnidentifiedFatalError,
+    PoorlyFormedMmessageHeader,
+    AttemptToUseConnectionWithoutBothChannels,
+    InvalidInitializationSequence,
+    ServerRefusedConnection
   } Fatal_Erro_code_t;
   static const char *Fatal_Error_Messages[] =
     {
@@ -185,7 +254,6 @@ namespace nsHiSLIP{
     };
     message_parameter(u_int16_t proto, u_int16_t sid){
       this->word = (proto << 16) + sid;
-
     };
     u_int16_t getServerProtocolVersion(){
       return (u_int16_t) (((this->word) & 0xffff0000)>>16);
@@ -202,10 +270,11 @@ namespace nsHiSLIP{
     u_int8_t  control_code;
     message_parameter_t message_parameter;
     u_int64_t payload_length;
-    
+    //
     Header(Message_Types_t message_type):control_code(0),message_parameter(0),payload_length(0){
       this->message_type=message_type;
     }
+    //
     Header(const void *buffer):message_parameter(0){
       assert(memcmp(this->prologue, buffer,2) == 0);
       this->message_type=*((u_int8_t *) ((char *) buffer+2));
@@ -213,6 +282,7 @@ namespace nsHiSLIP{
       this->message_parameter.word = ntohl(*(u_int32_t *) ((char *) buffer+4));
       this->payload_length = be64toh(*(u_int64_t *) ((char *) buffer+8));
     }
+    //
     Header(const u_int8_t  type,
 	   const u_int8_t  cce,
 	   const message_parameter_t param,
@@ -221,50 +291,40 @@ namespace nsHiSLIP{
       this->control_code=cce;
       this->payload_length=length;
     }
-    void print(void){
+    //
+    void print(const char *msg=NULL){
+      if (msg != NULL){
+	::printf("Header dump for: %s\n", msg);
+      }
       ::printf("message type:%d\n",this->message_type);
       ::printf("control_code:%d\n",this->control_code);
       ::printf("message_parameter: 0x%0x\n",this->message_parameter.word);
       ::printf("payload length: %llu\n",this->payload_length);
     }
-    
-    size_t send(int socket){
+    //
+    ssize_t send(int socket){
       char hbuf[HEADER_SIZE];
       ssize_t ssize;
       
       this->toRawData(hbuf);
-      ssize=::send(socket, hbuf, sizeof(hbuf), 0);
       
+      //ssize=::send(socket, hbuf, sizeof(hbuf), 0);
+      {
+	auto fut=std::async(std::launch::async,
+			    ::send,
+			    socket,hbuf,sizeof(hbuf),0);
+	ssize= fut.get();
+      }
       return ssize;
     }
-    
-    size_t recv(int socket){
+    //
+    ssize_t recv(int socket){
       unsigned char buffer[HEADER_SIZE];
-      return this->recv(socket,buffer);
+      return this->recv(socket, buffer);
     }
-    
-    size_t recv(int socket, void *buffer){
-      ssize_t rsize;
-      //rsize= ::recv(socket, buffer, HEADER_SIZE, 0);
-      rsize = ::recvfrom(socket, buffer, HEADER_SIZE, 0, NULL, NULL);
-      // rsize= ::read(socket, buffer, HEADER_SIZE);
+    //
+    ssize_t recv(int socket, void *buffer);
 
-      if (rsize < HEADER_SIZE){
-	//raise exception?
-	return -1;
-      }
-      
-      if (memcmp(this->prologue, buffer, 2) != 0){
-	return -1;
-      }
-      this->message_type=*((u_int8_t *) ((char *) buffer+2));
-      this->control_code=*((u_int8_t *) ((char *) buffer+3));
-      this->message_parameter.word = ntohl(*(u_int32_t *) ((char *) buffer+4));
-      this->payload_length = be64toh(*(u_int64_t *) ((char *) buffer+8));
-
-      return 0;
-    }
-    
     int fromRawData(void *buffer){ //DeSerialize
       if (memcmp(this->prologue, buffer, 2) != 0){
 	//error
@@ -289,8 +349,8 @@ namespace nsHiSLIP{
   
   typedef class Message:public Header{
   public:
-    void   *payload;
-    int    clean_on_exit;
+    void   *payload=NULL;
+    int    clean_on_exit=0;
     
     ~Message(){
       if ((this->payload != NULL) && (this->clean_on_exit==1)){
@@ -320,12 +380,14 @@ namespace nsHiSLIP{
 	    u_int8_t  cce,
 	    message_parameter_t param,
 	    u_int64_t length,
-	    u_int8_t *payload):Header(type,cce,param,length),payload(payload) {
+	    u_int8_t *payload):Header(type,
+				      cce,
+				      param,
+				      length),
+			       payload(payload) {
       this->clean_on_exit=0;
-      //this->payload= (void *) callocMustSucceed(1, length, "HiSLIP pyload buffer");
-      //memcpy(this->payload, payload, length);
-      //this->payload = payload;
     }
+    //
     size_t send(int socket){
       size_t ssize;
       // ssize=this->Header::send(socket);
@@ -341,142 +403,117 @@ namespace nsHiSLIP{
       return ssize;
     }
     
-    size_t recv(int socket, Message_Types_t expected_message_type = AnyMessages){
-      size_t rsize;
-      size_t status;
-      
-      status=this->Header::recv(socket);
-      if (status < 0){
-	// Error!
-	return status;
-      }
-
-      // now prepare for a pyload.
-      if (this->payload == NULL && this->payload_length > 0){
-	this->payload = (void *) calloc(this->payload_length,1);
-	if (this->payload == NULL){
-	  perror("faile to allocate memory for payload.");
-	  return -1;
-	}
-	this->clean_on_exit=1;
-      }
-      
-      rsize=0; //returns size of recieved payload. should be same as payload_length
-      if (this->payload_length > 0){
-	size_t bytestoread=this->payload_length;
-	
-	while (bytestoread){
-	  // status = ::recv(socket, ((u_int8_t *)this->payload+rsize),
-	  // 		  bytestoread, 0);
-	  status = ::recvfrom(socket, ((u_int8_t *)this->payload+rsize),
-	                            bytestoread, 0, NULL, NULL);
-	  // status = ::read(socket, ((u_int8_t *)this->payload+rsize),
-	  // 		  bytestoread);
-	  
-	  if (status <= 0){
-	    perror("payload read error:");
-	    return -1;
-	  }
-	  rsize += status;
-	  if (status >= bytestoread){
-		break;
-	  }
-	  bytestoread -=status;
-	}	  
-      }
-      // check if expected type or not
-      if((expected_message_type == AnyMessages) ||
-	 (expected_message_type == this->message_type)){
-	return 0;
-      }
-      // for debug
-      //this->print();
-      
-      if(this->message_type  == nsHiSLIP::Error){
-	::printf("Fatal Error: %d %s\n",
-		 this->control_code, nsHiSLIP::Error_Messages[this->control_code]);
-	if (this->payload_length >0){
-	  ::printf("Error msg: %s\n", (char *) this->payload);
-	}
-	return -(this->control_code+1);
-      }
-      else if(this->message_type  == nsHiSLIP::FatalError){
-	::printf("Error: %d %s\n",
-		 this->control_code, nsHiSLIP::Fatal_Error_Messages[this->control_code]);
-	if (this->payload_length >0){
-	  ::printf("Fatal Error msg: %s\n", (char *) this->payload);
-	}
-	return -(this->control_code+1);
-      }
-      return -1;
-    }
+    ssize_t recv(int socket, Message_Types_t expected_message_type = AnyMessages);
     
   } Message_t;
     
   typedef class HiSLIP {
   public:
-    unsigned long maximum_message_size; // max message size server accept
-    unsigned long maximum_payload_size;
-    unsigned long current_message_size; // current max message size setting
-    unsigned long current_payload_size; 
-    long socket_timeout;
-    long lock_timeout;
-    int sync_channel;
-    int async_channel;
+    unsigned long maximum_message_size = MAXIMUM_MESSAGE_SIZE; // max message size server accept
+    unsigned long maximum_payload_size = MAXIMUM_MESSAGE_SIZE;
+    unsigned long current_message_size = MAXIMUM_MESSAGE_SIZE - HEADER_SIZE; // current max message size setting
+    unsigned long current_payload_size = MAXIMUM_MESSAGE_SIZE - HEADER_SIZE; 
+    //
+    long socket_timeout = SOCKET_TIMEOUT;
+    long lock_timeout = LOCK_TIMEOUT;
+    int sync_channel = 0 ;
+    int async_channel = 0 ;
     struct pollfd sync_poll;
     struct pollfd async_poll;
-    int overlap_mode;
-    u_int8_t feature_setting;
+    //
+    u_int8_t feature_setting = 0;
+    u_int8_t feature_preference = 0 ;
     int session_id;
     int server_protocol_version;
     unsigned int server_vendorID;
-
-    bool rmt_delivered;
+    //mode variables for HiSLIP protocols.
+    bool overlap_mode = false; // false for synchronized mode, true for overlapped mode
+    bool interrupted = false;
+    bool async_interrupted = false;
+    bool rmt_delivered = false;
+    //
     u_int32_t message_id;
     u_int32_t most_recent_message_id;
+    u_int32_t most_recent_received_message_id;
     //sem_t srq_lock;
-    pthread_mutex_t srq_lock;
-    HiSLIP(){
-      this->maximum_message_size=this->current_message_size= MAXIMUM_MESSAGE_SIZE;
-      this->maximum_payload_size=this->current_payload_size= MAXIMUM_MESSAGE_SIZE - HEADER_SIZE;
-      this->socket_timeout=SOCKET_TIMEOUT;
-      this->lock_timeout=LOCK_TIMEOUT;
-      this->overlap_mode=false;
-      this->feature_setting=0;
-      this->rmt_delivered=false;
-      this->sync_channel=0;
-      this->async_channel=0;
-      this->srq_lock=PTHREAD_MUTEX_INITIALIZER;
-      // if (sem_init(&(this->srq_lock), 0, 1) !=0){
-      // 	perror(" HiSLIP srq_lock");
-      // }
-    };
+    pthread_mutex_t srq_lock=PTHREAD_MUTEX_INITIALIZER;
+    //
+    std::mutex srq_promise_mutex;
+    std::stack<std::promise<Message>> srq_promise_stack;
+    //
+    std::mutex srq_msg_mutex;
+    std::stack<Message> srq_msg_stack;
+    //
+    std::mutex promise_map_mutex;
+    std::map<Message_Types_t, std::promise<Message>> promise_map;
+    //
+    bool async_recceiver_active;
+    std::thread *async_receiver_thread=NULL;
+    //
+    ~HiSLIP(){
+      //::printf("destructor for HiSLIP\n");
+      this->stop_async_receiver_thread();
+      if (this->check_srq_lock()){
+	this->release_srq_lock();
+      }
+    }
+    //
     void connect(char const* hostname){
       this->connect(hostname,
 		    Default_device_name,
 		    Default_Port);
     };
+    
     void connect(char const* hostname,
 		 char const* dev_name,
-		 int  port);
-
+		 int  port); 
+    //
+    void start_async_receiver_thread(void);
+    void stop_async_receiver_thread(void){
+      this->async_recceiver_active=false;
+      if (this->async_receiver_thread !=NULL){
+	      this->async_receiver_thread->join();
+      };
+    };
+    void restart_async_receiver_thread(void){
+      this->stop_async_receiver_thread();
+      this->start_async_receiver_thread();
+    };
+    void async_receiver_loop(void);
+    std::future<Message> async_send_msg(Message msg,
+					Message_Types_t responseType=AnyMessages);
+    void register_async_promise(Message_Types_t responseType,
+				std::promise<Message> p);
+    void remove_async_promise(Message_Types_t responseType);
+    //
+    void push_srq_promise(std::promise<Message> p);
+    std::promise<Message> pop_srq_promise(void);
+    void clear_srq_promise(void);
+    //
+    void push_srq_msg(Message msg);
+    Message pop_srq_msg(void);
+    void clear_srq_msg(void);
+    void clear_srq_stacks(void);
+    //
     void set_timeout( long timeout){
       this->socket_timeout=timeout;
     };
+    
     long get_timeout(void){
       return this->socket_timeout;
     };
+    
     void set_lock_timeout( long timeout){
       this->lock_timeout=timeout;
-    }
-      ;
+    };
+    
     long get_lock_timeout(void){
       return this->lock_timeout;
     };
 
     long set_max_size(unsigned long message_size);
     int  device_clear(u_int8_t);
-    u_int8_t status_query(void);
+    int  status_query(void);
     //long write(u_int8_t *data_str, long timeout=LOCK_TIMEOUT);
     long write(const u_int8_t *data_str, const size_t size,
 	       long timeout=LOCK_TIMEOUT);
@@ -487,53 +524,59 @@ namespace nsHiSLIP{
 	      long timeout=LOCK_TIMEOUT);
     long read(size_t *received, u_int8_t *buffer, size_t bsize,
 	      long timeout=LOCK_TIMEOUT);
+    
+    long async_ask(u_int8_t *data_str, size_t size, long wait_time=LOCK_TIMEOUT);
+    long async_read( size_t *received, u_int8_t *buffer, size_t bsize,
+		     long timeout=LOCK_TIMEOUT);
+    
     long trigger_message(void);
     long remote_local(u_int8_t request);
-    long request_lock(const char* lock_string=NULL);
+    long request_lock(const char* lock_string = NULL);
     long release_lock(void);
+    long lock_info(void);
+    long handle_error_msg(void);
+    //
     long request_srq_lock(void);
     long release_srq_lock(void);
     int  check_srq_lock(void);
     int  check_and_lock_srq_lock(void);
-    u_int8_t get_Service_Request(void){
-      Message *msg=new Message(nsHiSLIP::AnyMessages);
-      long status;
-      
-      status= msg->recv(this->async_channel, nsHiSLIP::AsyncServiceRequest);
-
-      // for debug
-      //msg->print();
-      
-      if ((status & 0x80000000) != 0){
-	// should handle Error/Fatal Error/Async Interrupted messages.
-	perror(__FUNCTION__);
-      }
-      this->release_srq_lock();
-      return msg->control_code;
-    };
-    
-    int wait_for_SRQ(int wait_time){
+    //
+    int  get_Service_Request(void);
+    int  wait_Service_Request(int wait_time);
+    //
+    int wait_for_Async(int wait_time){
+      // just used in EPICS device support.
+      // shoudl use  promise-future.
+      // create promise and retister it to async-promise list.
+      // then wait future.
       this->async_poll.revents=0;
       return ::poll(&this->async_poll,  1,   wait_time);
     }
     
     void disconnect(){
-      if (this->sync_channel){
+      if (this->sync_channel != 0){
 	close(this->sync_channel);
+	this->sync_channel=0;
       };
-      if (this->async_channel){
+      if (this->async_channel != 0){
 	close(this->async_channel);
+	this->async_channel=0;
       };
     };
+
+    void Fatal_Error_handler(Message *msg);
+    void Error_handler(Message *msg);
+    void report_Fatal_Error(const u_int8_t erc, const char *errmsg);
+    void report_Error(const u_int8_t erc, const char *errmsg);
+    //
     
   private:
-    int wait_for_answer(int wait_time){
-      return ::poll(&this->sync_poll,  1,   wait_time);
-    }
     void reset_message_id(void){
       this->most_recent_message_id=INITIAL_LAST_MESSAGE_ID;
+      this->most_recent_received_message_id=INITIAL_MESSAGE_ID;
       this->message_id = INITIAL_MESSAGE_ID;
     }
+
     u_int32_t increment_message_id(void){
       this->most_recent_message_id=this->message_id;
       this->message_id = (this->message_id +2) & 0xffffffff;
